''' <summary>
''' Eine Klasse zum Halten einer Originalfarbe und einer abgewandelten Farbe
''' </summary>
Public Class AHSLColor

#Region " Private Felder "

    ' Transparenz der Originalfarbe
    Private m_A As Byte
    ' Originalfarbe
    Private m_OriginColor As HSLColor
    ' abgewandelte Farbe
    Private m_StretchedColor As HSLColor
    ' Abweichung des Farbtons bei 100% Dehnung vom mittleren Wert aller Farben der ColorTable
    Private m_HueDistance As Decimal

#End Region

#Region " Konstruktoren "

    ''' <summary>
    ''' Konstruktor mit einer Originalfarbe
    ''' </summary>
    ''' <param name="color">Color-Struktur</param>
    Public Sub New(ByVal color As Color)
        ' Transparenz
        m_A = color.A
        ' HSL fr Originalfarbe erstellen
        m_OriginColor = New HSLColor(color)
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' Farbwert der Originalfarbe
    ''' </summary>
    ReadOnly Property Hue() As Double
        Get
            Return m_OriginColor.Hue
        End Get
    End Property

    ''' <summary>
    ''' Farbwert der durch Dehnung um StrechFactor resultierenden Farbe 
    ''' </summary>
    ReadOnly Property StretchedHue() As Double
        Get
            Return m_StretchedColor.Hue
        End Get
    End Property

#End Region

#Region " Methoden "

    ''' <summary>
    ''' Berechnet die Distanz zwischen MidHue und Hue-Wert der Basisfarbe
    ''' </summary>
    ''' <param name="MidHue">Mittlerer Farbwert</param>
    ''' <param name="StartStretchFactor">Dehnungsfaktor</param>
    Public Sub CalculateHueDistance(ByVal MidHue As Decimal, ByVal StartStretchFactor As Decimal)
        ' Die Division durch den StretchFactor bewirkt, dass die berechnete Distanz 
        ' fr einen StretchFactor 1 gltig ist
        ' Fr das Berechnen dieser Farbdistanz fr einen anderen Dehnungsfaktor 
        ' muss HueDistance also nur mit dem neuen Dehnungsfaktor multipliziert werden
        m_HueDistance = (CDec(m_OriginColor.Hue) - MidHue) / StartStretchFactor
    End Sub

    ''' <summary>
    ''' erstellt eine resultierende Farbe unter Verwendung eines mittleren Farbwertes und eines Dehnungsfaktors
    ''' </summary>
    ''' <param name="MidHue">mittlerer Farbwert</param>
    ''' <param name="StretchFactor">Dehnungsfaktor</param>
    Public Sub GetStretchedColor(ByVal MidHue As Decimal, ByVal StretchFactor As Decimal)
        m_StretchedColor = New HSLColor(m_HueDistance * StretchFactor + MidHue, m_OriginColor.Saturation, m_OriginColor.Lightness)
    End Sub

    ''' <summary>
    ''' Anzuzeigende Farbe nach Dehnung und Farbwert-Verschiebung
    ''' </summary>
    ''' <returns>Color-Struktur</returns>
    Public Function DisplayColor() As Color
        Return Color.FromArgb(m_A, m_StretchedColor.ToRGB)
    End Function

#End Region

End Class
