''' <summary>
''' Ein Control zum Anzeigen und Definieren von Hue und StretchFactor 
''' fr eine beliebige ProfessionalColorTable
''' </summary>
Public Class ColorWheelProfessionalTableControl
    Inherits Control

#Region " Private Felder / Events "

    ' zugrunde liegende Farbtabelle
    Private m_Table As ProfessionalColorTable

    ' mittlerer Farbwert
    Private m_MidHue As Decimal

    ' aktuell ausgewhlter Farbwert
    Private m_Hue As Double = 0.0R
    ' aktuell ausgewhlter Dehnungsfaktor
    Private m_StretchFactor As Decimal = 0D

    ' Hintergrund-Bitmap, das eine Farbauswahl aus der Farbtabelle anzeigt
    Private m_BackBitmap As Bitmap

    ' Beschriftung
    Private m_LeftText As String = "Button Color"
    Private m_BottomText As String = "Control Color"
    Private m_LeftTextPosition As Point
    Private m_BottomTextPosition As Point

    ' aktuell ausgewhlte Marker-Position im Farbraster
    Private m_MarkerPosition As Point

    ''' <summary>
    ''' tritt ein, wenn Farbwert oder Dehnungsfaktor vom Control aus gendert wurden
    ''' </summary>
    Public Event Changed()

    ' hier verwendete benannte Farben
    Private m_ButtonCheckedGradientEnd As AHSLColor
    Private m_ToolStripGradientMiddle As AHSLColor

    ' Rechteck mit Position und Gre der Hintergrund-Bitmap 
    Private m_ControlRectangle As New Rectangle(20, 10, 128, 128)
    ' True, wenn linker Mousebutton ber dem ControlRectangle bedient wurde
    Private m_IsMouseDown As Boolean = False

#End Region

#Region " Konstruktoren "

    ''' <summary>
    ''' Konstruktor ohne Parameter
    ''' </summary>
    Public Sub New()
        ' Basis-Farbtabelle definieren
        m_Table = New ExampleColorTable
        ' Basisdaten aus Farbtabelle ermitteln
        GetColors_First()
        ' Minimalgre des Controls festlegen
        Me.MinimumSize = New Size(158, 158)
        ' Positionen fr zu zeichnende Textabschnitte ermitteln
        GetTextPositions()
        ' MarkerPosition fr Start-Farbwert und -Dehnungsfaktor ermitteln
        m_MarkerPosition = GetMarkerPosition(m_Hue, m_StretchFactor)
        ' DoubleBuffering aktivieren
        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint _
         Or ControlStyles.AllPaintingInWmPaint, True)
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' Farbwert
    ''' </summary>
    ''' <remarks>Ein Wert zwischen 0 und 1</remarks>
    Property Hue() As Double
        Get
            Return m_Hue
        End Get
        Set(ByVal value As Double)
            Dim NewHue As Double = Math.Max(0.0R, Math.Min(1.0R, value))
            If m_Hue <> NewHue Then
                ' Wert bernehmen
                m_Hue = NewHue
                ' Anzeige aktualisieren
                RefreshMarkerPosition()
            End If
        End Set
    End Property

    ''' <summary>
    ''' Dehnungsfaktor
    ''' </summary>
    ''' <remarks>Ein Wert zwischen -1 und +1</remarks>
    Property StretchFactor() As Decimal
        Get
            Return m_StretchFactor
        End Get
        Set(ByVal value As Decimal)
            Dim NewStretchFactor As Decimal = Math.Max(-1D, Math.Min(1D, value))
            If m_StretchFactor <> NewStretchFactor Then
                ' Wert bernehmen
                m_StretchFactor = NewStretchFactor
                ' Anzeige aktualisieren
                RefreshMarkerPosition()
            End If
        End Set
    End Property

    ''' <summary>
    ''' gibt die angezeigte Farbtabelle zurck oder legt diese fest
    ''' </summary>
    Property Table() As ProfessionalColorTable
        Get
            Return m_Table
        End Get
        Set(ByVal value As ProfessionalColorTable)
            If m_Table IsNot value AndAlso value IsNot Nothing Then
                m_Table = value
                ' Basisdaten aus der neuen Farbtabelle ermitteln
                GetColors_First()
                ' Hintergrund-Bitmap lschen
                m_BackBitmap = Nothing
                ' Control zum Neuzeichnen zwingen
                Invalidate()
            End If
        End Set
    End Property

#End Region

#Region " Methoden "

    ''' <summary>
    ''' ermittelt die Markerposition fr einen gegebenen Farbwert und Dehnungsfaktor
    ''' </summary>
    ''' <param name="Hue">Farbwert</param>
    ''' <param name="StretchFactor">Dehnungsfaktor</param>
    Private Function GetMarkerPosition(ByVal Hue As Double, ByVal StretchFactor As Decimal) As Point
        Dim LeftPos As Integer = CInt(Hue * 128.0R) + 20
        Dim TopPos As Integer = (StretchFactor * 64D) + 64 + 10
        Return New Point(LeftPos, TopPos)
    End Function

    ''' <summary>
    ''' Positionen fr im Control zu zeichnenden Text neu ermitteln
    ''' </summary>
    Private Sub GetTextPositions()
        ' Graphics-Objekt erstellen
        Using g As Graphics = Me.CreateGraphics
            ' Positionen fr Beschriftungen berechnen
            Dim Sz As SizeF = g.MeasureString(m_BottomText, Me.Font)
            m_BottomTextPosition = New Point(20 + 129 - Sz.Width, 10 + 129)

            Sz = g.MeasureString(m_LeftText, Me.Font)
            m_LeftTextPosition = New Point(20 - Sz.Height, 10 + Sz.Width)
        End Using
    End Sub

    ''' <summary>
    ''' ermittelt die zur Anzeige notwendigen Basisdaten aus einer Farbtabelle neu
    ''' </summary>
    Private Sub GetColors_First()
        ' Liste mit allen in der Farbtabelle enthaltenen Farben erstellen
        Dim AllColors As New List(Of AHSLColor)

        AllColors.Add(New AHSLColor(m_Table.ButtonCheckedGradientBegin))
        AllColors.Add(New AHSLColor(m_Table.ButtonCheckedGradientMiddle))
        AllColors.Add(New AHSLColor(m_Table.ButtonCheckedHighlight))
        AllColors.Add(New AHSLColor(m_Table.ButtonCheckedHighlightBorder))
        AllColors.Add(New AHSLColor(m_Table.ButtonPressedBorder))
        AllColors.Add(New AHSLColor(m_Table.ButtonPressedGradientBegin))
        AllColors.Add(New AHSLColor(m_Table.ButtonPressedGradientEnd))
        AllColors.Add(New AHSLColor(m_Table.ButtonPressedGradientMiddle))
        AllColors.Add(New AHSLColor(m_Table.ButtonPressedHighlight))
        AllColors.Add(New AHSLColor(m_Table.ButtonPressedHighlightBorder))
        AllColors.Add(New AHSLColor(m_Table.ButtonSelectedBorder))
        AllColors.Add(New AHSLColor(m_Table.ButtonSelectedGradientBegin))
        AllColors.Add(New AHSLColor(m_Table.ButtonSelectedGradientEnd))
        AllColors.Add(New AHSLColor(m_Table.ButtonSelectedGradientMiddle))
        AllColors.Add(New AHSLColor(m_Table.ButtonSelectedHighlight))
        AllColors.Add(New AHSLColor(m_Table.ButtonSelectedHighlightBorder))
        AllColors.Add(New AHSLColor(m_Table.CheckBackground))
        AllColors.Add(New AHSLColor(m_Table.CheckPressedBackground))
        AllColors.Add(New AHSLColor(m_Table.CheckSelectedBackground))
        AllColors.Add(New AHSLColor(m_Table.GripDark))
        AllColors.Add(New AHSLColor(m_Table.GripLight))
        AllColors.Add(New AHSLColor(m_Table.ImageMarginGradientBegin))
        AllColors.Add(New AHSLColor(m_Table.ImageMarginGradientEnd))
        AllColors.Add(New AHSLColor(m_Table.ImageMarginGradientMiddle))
        AllColors.Add(New AHSLColor(m_Table.ImageMarginRevealedGradientBegin))
        AllColors.Add(New AHSLColor(m_Table.ImageMarginRevealedGradientEnd))
        AllColors.Add(New AHSLColor(m_Table.ImageMarginRevealedGradientMiddle))
        AllColors.Add(New AHSLColor(m_Table.MenuBorder))
        AllColors.Add(New AHSLColor(m_Table.MenuItemBorder))
        AllColors.Add(New AHSLColor(m_Table.MenuItemPressedGradientBegin))
        AllColors.Add(New AHSLColor(m_Table.MenuItemPressedGradientEnd))
        AllColors.Add(New AHSLColor(m_Table.MenuItemPressedGradientMiddle))
        AllColors.Add(New AHSLColor(m_Table.MenuItemSelected))
        AllColors.Add(New AHSLColor(m_Table.MenuItemSelectedGradientBegin))
        AllColors.Add(New AHSLColor(m_Table.MenuItemSelectedGradientEnd))
        AllColors.Add(New AHSLColor(m_Table.MenuStripGradientBegin))
        AllColors.Add(New AHSLColor(m_Table.MenuStripGradientEnd))
        AllColors.Add(New AHSLColor(m_Table.OverflowButtonGradientBegin))
        AllColors.Add(New AHSLColor(m_Table.OverflowButtonGradientEnd))
        AllColors.Add(New AHSLColor(m_Table.OverflowButtonGradientMiddle))
        AllColors.Add(New AHSLColor(m_Table.RaftingContainerGradientBegin))
        AllColors.Add(New AHSLColor(m_Table.RaftingContainerGradientEnd))
        AllColors.Add(New AHSLColor(m_Table.SeparatorDark))
        AllColors.Add(New AHSLColor(m_Table.SeparatorLight))
        AllColors.Add(New AHSLColor(m_Table.StatusStripGradientBegin))
        AllColors.Add(New AHSLColor(m_Table.StatusStripGradientEnd))
        AllColors.Add(New AHSLColor(m_Table.ToolStripBorder))
        AllColors.Add(New AHSLColor(m_Table.ToolStripContentPanelGradientBegin))
        AllColors.Add(New AHSLColor(m_Table.ToolStripContentPanelGradientEnd))
        AllColors.Add(New AHSLColor(m_Table.ToolStripDropDownBackground))
        AllColors.Add(New AHSLColor(m_Table.ToolStripGradientBegin))
        AllColors.Add(New AHSLColor(m_Table.ToolStripPanelGradientBegin))
        AllColors.Add(New AHSLColor(m_Table.ToolStripPanelGradientEnd))
        AllColors.Add(New AHSLColor(m_Table.ToolStripGradientEnd))

        ' ButtonCheckedGradientEnd und ToolStripGradientMiddle (feste Werte aus der Farbtabelle)
        ' werden zum Berechnen der Anzeige immer wieder gebraucht
        ' Anmerkung: Das Control zeigt auf der x-Achse den Hue-Wert fr ToolStripGradientMiddle an
        ' ToolStripGradientMiddle ist in diesem Beispiel die fr Controls verwendete Hintergrundfarbe
        ' alle anderen Farben hngen davon ab.
        ' nderungen auf der y-Achse bewirken eine nderung des Hue-Wertes von ButtonCheckedGradientEnd
        m_ButtonCheckedGradientEnd = New AHSLColor(m_Table.ButtonCheckedGradientEnd)
        m_ToolStripGradientMiddle = New AHSLColor(m_Table.ToolStripGradientMiddle)

        AllColors.Add(m_ButtonCheckedGradientEnd)
        AllColors.Add(m_ToolStripGradientMiddle)

        ' Minimum und Maximum fr Hue ermitteln
        Dim MinHue As Double = Double.MaxValue
        Dim MaxHue As Double = Double.MinValue

        For Each col As AHSLColor In AllColors
            If col.Hue < MinHue Then
                MinHue = col.Hue
            End If
            If col.Hue > MaxHue Then
                MaxHue = col.Hue
            End If
        Next


        ' mittlerer Wert fr Hue
        m_MidHue = (MaxHue + MinHue) / 2D

        ' Dehnungsfaktor fr die Basis-Tabelle
        Dim StretchFactor As Decimal = MaxHue - MinHue
        m_ButtonCheckedGradientEnd.CalculateHueDistance(m_MidHue, StretchFactor)
        m_ToolStripGradientMiddle.CalculateHueDistance(m_MidHue, StretchFactor)

    End Sub

    ''' <summary>
    ''' liefert ButtonCheckedGradientEndColor fr einen beliebigen Farb- und Dehnungsfaktor
    ''' </summary>
    ''' <param name="Hue">Farbwert zwischen 0 und 1</param>
    ''' <param name="StretchFactor">Dehnungsfaktor zwischen -1 und +1</param>
    Private Function GetButtonCheckedGradientEndColor(ByVal Hue As Double, ByVal StretchFactor As Decimal) As Color

        ' probeweise "gedehnte" Farbe fr angegebenen Dehnungsfaktor und bekannten Farben-Mittelwert ermitteln
        m_ToolStripGradientMiddle.GetStretchedColor(m_MidHue, StretchFactor)

        ' Um die gewnschte Farbe fr ToolStripGradientMiddle zu erreichen, muss der Farb-Mittelwert
        ' fr alle Zielfarben verndert werden
        Dim HueShift As Decimal = m_ToolStripGradientMiddle.StretchedHue - Hue
        Dim NewMidHue As Decimal = m_MidHue - HueShift

        ' Zielfarben mit angegebenem Dehnungsfaktor und neuem Farb-Mittelwert ermitteln
        m_ButtonCheckedGradientEnd.GetStretchedColor(NewMidHue, StretchFactor)
        m_ToolStripGradientMiddle.GetStretchedColor(NewMidHue, StretchFactor)

        ' Farbe zurck geben
        Return m_ButtonCheckedGradientEnd.DisplayColor
    End Function

    ''' <summary>
    ''' liefert ToolStripGradientMiddleColor fr einen beliebigen Farb- und Dehnungsfaktor
    ''' </summary>
    ''' <param name="Hue">Farbwert zwischen 0 und 1</param>
    ''' <param name="StretchFactor">Dehnungsfaktor zwischen -1 und +1</param>
    Private Function GetToolStripGradientMiddleColor(ByVal Hue As Double, ByVal StretchFactor As Decimal) As Color

        ' probeweise "gedehnte" Farbe fr angegebenen Dehnungsfaktor und bekannten Farben-Mittelwert ermitteln
        m_ToolStripGradientMiddle.GetStretchedColor(m_MidHue, StretchFactor)

        ' Um die gewnschte Farbe fr ToolStripGradientMiddle zu erreichen, muss der Farb-Mittelwert
        ' fr alle Zielfarben verndert werden
        Dim HueShift As Decimal = m_ToolStripGradientMiddle.StretchedHue - Hue
        Dim NewMidHue As Decimal = m_MidHue - HueShift

        ' Zielfarben mit angegebenem Dehnungsfaktor und neuem Farb-Mittelwert ermitteln
        m_ButtonCheckedGradientEnd.GetStretchedColor(NewMidHue, StretchFactor)
        m_ToolStripGradientMiddle.GetStretchedColor(NewMidHue, StretchFactor)

        ' Farbe zurck geben
        Return m_ToolStripGradientMiddle.DisplayColor
    End Function

    ''' <summary>
    ''' berschreiben der OnPaint-Methode zum Zeichnen des Controls
    ''' </summary>
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)
        Dim g As Graphics = e.Graphics

        ' Bei Notwendigkeit Hintergrund-Bitmap neu erstellen
        If m_BackBitmap Is Nothing Then CreateBackBitmap()

        ' Hintergrund-Bitmap zeichnen
        g.DrawImage(m_BackBitmap, New Point(20, 10))

        ' Beschriftungen zeichnen
        g.DrawString(m_BottomText, Me.Font, Brushes.Black, m_BottomTextPosition)
        SharedFunctions.DrawRotatedText(g, m_LeftTextPosition, 270, m_LeftText, Me.Font, Brushes.Black)

        ' Marker zeichnen
        Dim SymbolRect As New RectangleF(m_MarkerPosition.X - 5, m_MarkerPosition.Y - 5, 10, 10)
        g.DrawEllipse(Pens.Black, SymbolRect)
        SymbolRect.Inflate(-1.0F, -1.0F)
        g.SmoothingMode = Drawing2D.SmoothingMode.AntiAlias
        g.DrawEllipse(Pens.White, SymbolRect)

    End Sub


    ''' <summary>
    ''' berschreiben der OnMouseDown-Methode der Control-Basisklasse
    ''' </summary>
    Protected Overrides Sub OnMouseDown(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseDown(e)
        ' wenn nicht linker Mousebutton bettigt wurde, keine weitere Aktion
        If e.Button <> Windows.Forms.MouseButtons.Left Then Exit Sub
        ' wenn sich die Maus im Control-Bereich befindet
        If m_ControlRectangle.Contains(e.Location) Then
            ' Flag fr MouseDown auf True setzen
            m_IsMouseDown = True
            ' Marker zur MousePosition bewegen und eventuell Event fr nderung auslsen
            MoveMarkerToPosition(e.Location)
        End If
    End Sub

    ''' <summary>
    ''' berschreiben der OnMouseUp-Methode der Control-Basisklasse
    ''' </summary>
    Protected Overrides Sub OnMouseUp(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseUp(e)
        ' Flag fr MouseDown auf False setzen
        m_IsMouseDown = False
    End Sub

    ''' <summary>
    ''' berschreiben der OnMouseMove-Methode der Control-Basisklasse
    ''' </summary>
    Protected Overrides Sub OnMouseMove(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseMove(e)
        ' nur wenn MouseDown-Flag True ist, d.h., linker MouseButton wurde innerhalb des Control-Bereiches gedrckt
        If m_IsMouseDown Then
            ' Marker zur MousePosition bewegen und eventuell Event fr nderung auslsen
            MoveMarkerToPosition(e.Location)
        End If
    End Sub

    ''' <summary>
    ''' bewegt den Marker zu einem Punkt innerhalb des Control-Rechtecks
    ''' </summary>
    ''' <param name="MousePosition">Ziel-Position</param>
    Private Sub MoveMarkerToPosition(ByVal MousePosition As Point)
        ' Die X- und Y- Koordinate drfen nur innerhalb des festgelegenen Control-Bereiches liegen
        Dim X As Integer = Math.Min(Math.Max(m_ControlRectangle.X, MousePosition.X), m_ControlRectangle.Right)
        Dim Y As Integer = Math.Min(Math.Max(m_ControlRectangle.Y, MousePosition.Y), m_ControlRectangle.Bottom)

        ' nur wenn die neuen Koordinaten gegenber den bisherigen gendert wurden
        If m_MarkerPosition.X <> X OrElse m_MarkerPosition.Y <> Y Then

            ' altes Marker-Rechteck
            Dim OldMarkerRect As New Rectangle(m_MarkerPosition.X - 6, m_MarkerPosition.Y - 6, 12, 12)
            ' Marker-Position bernehmen
            m_MarkerPosition = New Point(X, Y)
            ' neues Marker-Rechteck
            Dim NewMarkerRect As New Rectangle(m_MarkerPosition.X - 6, m_MarkerPosition.Y - 6, 12, 12)

            ' neue Werte fr Hue und Dehnungsfaktor aus Marker-Position ermitteln
            Dim NewHue As Double = CDbl(X - 20) / 128.0R
            Dim NewStretchFactor As Decimal = CDec(Y - 10 - 64) / 64D
            ' Wenn sich Hue und Dehnungsfaktor gegenber den bisher bekannten gendert haben
            If m_Hue <> NewHue OrElse m_StretchFactor <> NewStretchFactor Then
                ' neue Werte fr Hue und Dehnungsfaktor bernehmen
                m_Hue = NewHue
                m_StretchFactor = NewStretchFactor
                ' nderungs-Event auslsen
                RaiseEvent Changed()
            End If

            ' Control zum Neuzeichnen nur der genderten Marker-Flchen zwingen
            Invalidate(Rectangle.Union(OldMarkerRect, NewMarkerRect))
        End If

    End Sub

    ''' <summary>
    ''' bewirkt Aktualisierung des Markers nach nderung von Hue oder Dehnungsfaktor ber die Properties
    ''' </summary>
    Private Sub RefreshMarkerPosition()
        ' neue Marker-Position fr bekannten Farbwert und Dehnungsfaktor ermitteln
        Dim NewMarkerPosition As Point = GetMarkerPosition(m_Hue, m_StretchFactor)

        ' nur wenn die neue Marker-Position sich von der bekannten unterscheidet
        If m_MarkerPosition.X <> NewMarkerPosition.X OrElse m_MarkerPosition.Y <> NewMarkerPosition.Y Then
            ' altes Marker-Rechteck
            Dim OldMarkerRect As New Rectangle(m_MarkerPosition.X - 6, m_MarkerPosition.Y - 6, 12, 12)
            ' Marker-Position bernehmen
            m_MarkerPosition = NewMarkerPosition
            ' neues Marker-Rechteck
            Dim NewMarkerRect As New Rectangle(m_MarkerPosition.X - 6, m_MarkerPosition.Y - 6, 12, 12)

            ' Control zum Neuzeichnen nur der genderten Marker-Flchen zwingen
            Invalidate(Rectangle.Union(OldMarkerRect, NewMarkerRect))
        End If

    End Sub

    ''' <summary>
    ''' Erstellt Bitmap zum Anzeigen der Farbskala
    ''' </summary>
    Private Sub CreateBackBitmap()
        m_BackBitmap = New Bitmap(129, 129, Imaging.PixelFormat.Format32bppArgb)

        ' Graphics-Objekt aus Bitmap erstellen
        Using g As Graphics = Graphics.FromImage(m_BackBitmap)

            ' Pen zum Zeichnen von Linien
            Dim P As New Pen(Color.White)

            ' Hintergrund mit senkrechten Linien fr unterschiedliche Control-Farben
            ' zeichnen (als Control-Farbe verwende ich ToolStripGradientMiddle aus der ColorTable)
            For I As Integer = 0 To 128
                P.Color = GetToolStripGradientMiddleColor(CDbl(I) / 128.0R, 0D)
                g.DrawLine(P, I, 0, I, 128)
            Next

            ' im Vordergrund waagerechte Linien mit Farbverlauf fr entsprechende Button-Farben zeichnen
            For J As Integer = 0 To 128
                Dim Hue As Double = CDbl(J) / 128.0R
                For I As Integer = 0 To 128 Step 8
                    P.Color = GetButtonCheckedGradientEndColor(Hue, CDec(I - 64) / 64D)
                    g.DrawLine(P, J, I, J + 1, I)
                Next
            Next

            ' Pen-Ressourcen frei geben
            P.Dispose()
        End Using
    End Sub

#End Region

End Class
