''' <summary>
''' Eine generische unsortierte Liste, die absichert, dass immer genau ein Item 
''' ausgewhlt ist
''' </summary>
Public MustInherit Class AddRemoveSelectionList(Of T)
    Inherits List(Of T)
    ' diese Listen-Hierarchie habe ich mir berlegt, um im ListPaintControl
    ' entweder sortierte oder auch unsortierte Listen verwenden zu knnen

    ' tritt ein, wenn sich die Anzahl der ListItems (und ggf. auch SelectedItem) gendert hat
    Public Event Changed()
    ' tritt ein, wenn nur SelectedItem gendert wurde
    Public Event SelectedItemChanged()
    Friend m_SelectedItem As T = Nothing
    MustOverride Function IsSame(ByVal Value1 As T, ByVal Value2 As T) As Boolean

    ''' <summary>
    ''' Ausgewhltes Item
    ''' </summary>
    Property SelectedItem() As T
        Get
            Return m_SelectedItem
        End Get
        Set(ByVal value As T)
            If Not IsSame(value, m_SelectedItem) Then
                If value Is Nothing OrElse Me.Contains(value) Then
                    m_SelectedItem = value
                    RaiseEvent SelectedItemChanged()
                End If
            End If
        End Set
    End Property

    ''' <summary>
    ''' Ausgewhlter Index
    ''' </summary>
    Property SelectedIndex() As Integer
        Get
            If m_SelectedItem Is Nothing Then
                Return -1
            Else
                Return Me.IndexOf(m_SelectedItem)
            End If
        End Get
        Set(ByVal value As Integer)
            Dim NewSelected As T = Nothing
            If value > -1 AndAlso value < Me.Count Then
                NewSelected = Me(value)
            End If
            SelectedItem = NewSelected
        End Set
    End Property

    Public Shadows Sub Add(ByVal item As T)
        MyBase.Add(item)
        m_SelectedItem = item
        RaiseEvent Changed()
    End Sub
    Public Shadows Sub AddRange(ByVal collection As IEnumerable(Of T))
        Dim TmpListe As New List(Of T)
        TmpListe.AddRange(collection)
        MyBase.AddRange(collection)
        If TmpListe.Count > 0 Then
            m_SelectedItem = TmpListe(0)
            RaiseEvent Changed()
        End If
    End Sub
    Public Shadows Sub Clear()
        MyBase.Clear()
        m_SelectedItem = Nothing
        RaiseEvent Changed()
    End Sub
    Public Shadows Sub Insert(ByVal index As Integer, ByVal item As T)
        MyBase.Insert(index, item)
        m_SelectedItem = item
        RaiseEvent Changed()
    End Sub
    Public Shadows Sub InsertRange(ByVal index As Integer, ByVal collection As IEnumerable(Of T))
        MyBase.InsertRange(index, collection)
        Dim TmpListe As New List(Of T)
        TmpListe.AddRange(collection)
        If TmpListe.Count > 0 Then
            m_SelectedItem = TmpListe(0)
            RaiseEvent Changed()
        End If
    End Sub
    Public Shadows Function Remove(ByVal item As T) As Boolean
        Dim Index As Integer = Me.IndexOf(item)
        If Index > -1 Then
            Dim result As Boolean = MyBase.Remove(item)
            If Index > 0 Then
                m_SelectedItem = Me(Index - 1)
            ElseIf Me.Count > 0 Then
                m_SelectedItem = Me(0)
            Else
                m_SelectedItem = Nothing
            End If
            RaiseEvent Changed()
            Return result
        End If
        Return False
    End Function
    Public Shadows Sub RemoveAt(ByVal index As Integer)
        MyBase.RemoveAt(index)
        If index > 0 Then
            m_SelectedItem = Me(index - 1)
        ElseIf Me.Count > 0 Then
            m_SelectedItem = Me(0)
        Else
            m_SelectedItem = Nothing
        End If
        RaiseEvent Changed()
    End Sub

    ''' <summary>
    ''' erlaubt, erbenden Klassen, das Changed-Event auszulsen
    ''' </summary>
    Friend Sub OnChanged()
        RaiseEvent Changed()
    End Sub
End Class
