''' <summary>
''' Ein ListItem, das ein Image-Objekt und einen Text enthlt
''' </summary>
Public Class ImageTextItem
    Inherits ListItem

#Region " Private Felder "

    ' Grafik
    Private m_Image As Image = Nothing
    ' Text
    Private m_Text As String

#End Region

#Region " Konstruktoren "

    ''' <summary>
    ''' Konstruktor fr ein ListItem, mit Image-Objekt und Text
    ''' </summary>
    ''' <param name="Image">Image-Objekt</param>
    ''' <param name="Text">String</param>
    Public Sub New(ByVal Image As Image, ByVal Text As String)
        m_Image = Image
        m_Text = Text
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' gibt das Image-Objekt zurck, das diesem ListItem zugeordnet ist
    ''' </summary>
    ReadOnly Property Image() As Image
        Get
            Return m_Image
        End Get
    End Property
    ''' <summary>
    ''' gibt den Text zurck, der diesem ListItem zugeordnet ist
    ''' </summary>
    ReadOnly Property Text() As String
        Get
            Return m_Text
        End Get
    End Property

#End Region

#Region " Methoden "

    ''' <summary>
    ''' Erstellt ein ListPaintItem-Objekt, das gezeichnet werden kann
    ''' </summary>
    ''' <param name="Selected">True, wenn das ListItem selektiert ist</param>
    Public Overrides Function CreatePaintItem(ByVal Selected As Boolean) As ListPaintItem
        Return New ImageTextPaintItem(Create48PixelImage, m_Text, Selected)
    End Function
    ''' <summary>
    ''' Erstellt eine Bitmap mit 48 Pixel Kantenlnge als Vorschaugrafik
    ''' </summary>
    Private Function Create48PixelImage() As Image
        ' neue Bitmap erstellen
        Dim BMP As New Bitmap(48, 48, Drawing.Imaging.PixelFormat.Format32bppArgb)
        ' Graphics-Objekt aus Bitmap liefern
        Dim g As Graphics = Graphics.FromImage(BMP)
        If m_Image IsNot Nothing Then
            ' darauf das Image-Objekt in verkleinerter Gre zeichnen
            g.DrawImage(m_Image, New Rectangle(0, 0, 48, 48))
        End If
        Return BMP
    End Function

#End Region

End Class
